package com.smppw.analysis.domain.dao;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dataobject.*;
import com.smppw.analysis.domain.dataobject.info.FundInfoDO;
import com.smppw.analysis.domain.mapper.core.*;
import com.smppw.common.pojo.ValueLabelVO;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.Indicator;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class FundInformationDao {
    private final FundAssetSizeMapper fundAssetSizeMapper;
    private final FundFrequencyMapper fundFrequencyMapper;
    private final FundInformationMapper fundInformationMapper;
    private final MonetaryFundProfitMapper monetaryFundProfitMapper;
    private final CompanyFundScaleRangeHistoryMapper companyFundScaleRangeHistoryMapper;

    public FundInformationDao(FundAssetSizeMapper fundAssetSizeMapper,
                              FundFrequencyMapper fundFrequencyMapper,
                              FundInformationMapper fundInformationMapper,
                              MonetaryFundProfitMapper monetaryFundProfitMapper,
                              CompanyFundScaleRangeHistoryMapper companyFundScaleRangeHistoryMapper) {
        this.fundAssetSizeMapper = fundAssetSizeMapper;
        this.fundFrequencyMapper = fundFrequencyMapper;
        this.fundInformationMapper = fundInformationMapper;
        this.monetaryFundProfitMapper = monetaryFundProfitMapper;
        this.companyFundScaleRangeHistoryMapper = companyFundScaleRangeHistoryMapper;
    }

    /**
     * 获取多个基金的基本信息
     *
     * @param fundIds /
     * @return /
     */
    public List<FundInfoDO> listFundInfo(List<String> fundIds) {
        return this.fundInformationMapper.queryFundInfoList(fundIds);
    }

    public FundInfoDO getFundInfoById(String fundId) {
        List<FundInfoDO> dataList = this.fundInformationMapper.queryFundInfoList(ListUtil.toList(fundId));
        return dataList.stream().filter(e -> fundId.equals(e.getFundId())).findFirst().orElse(null);
    }

    public String getNavFrequencyByFundId(String secId) {
        return this.fundFrequencyMapper.getNavFrequencyByFundId(secId);
    }

    public Map<String, Frequency> getNavFrequency(List<String> secIdList) {
        Map<String, Frequency> secFrequencyMap = new HashMap<>();
        List<FundFrequencyDo> fundFrequencyDos = fundFrequencyMapper.getNavFrequencyByFundIdList(secIdList);
        for (FundFrequencyDo fundFrequencyDo : fundFrequencyDos) {
            String frequencyStr = fundFrequencyDo.getNavFrequency();
            Frequency frequency = Frequency.Monthly;
            if ("天".equals(frequencyStr)) {
                frequency = Frequency.Daily;
            } else if ("周".equals(frequencyStr)) {
                frequency = Frequency.Weekly;
            }
            secFrequencyMap.put(fundFrequencyDo.getFundId(), frequency);
        }
        return secFrequencyMap;
    }

    public Map<String, String> getMarketFundIdNameMap(List<String> fundIdList) {
        List<FundInformationDo> fundInformationDos = fundInformationMapper.listFundInfo(fundIdList);
        Map<String, String> fundIdNameMap = new HashMap<>();
        for (FundInformationDo fundInformationDo : fundInformationDos) {
            fundIdNameMap.put(fundInformationDo.getFundId(), fundInformationDo.getFundShortName());
        }
        return fundIdNameMap;
    }

    public List<MonetaryFundProfitDO> queryMonetaryFund(String fundId) {
        return this.monetaryFundProfitMapper.queryByFundId(fundId);
    }

    public List<Map<String, Object>> getFundRank(String rankDate, String fundId, List<String> indexIds, Indicator indicator) {
        Map<String, Object> params = MapUtil.newHashMap(8);
        params.put("fundId", fundId);
        params.put("rankDate", rankDate);
        params.put("indexIds", String.join("_", indexIds));
        params.put("indicator", indicator.getId());
        return this.fundInformationMapper.getFundRankByCall(params);
    }

    public List<FundSimilarDo> getSameCompanyFundIds(Map<String, Object> params) {
        return this.fundInformationMapper.getSameCompanyFundIds(params);
    }

    public List<FundSimilarDo> getSameStrategyFundIds(Map<String, Object> params) {
        return this.fundInformationMapper.getSameStrategyFundIds(params);
    }

    public FundFeeDo getFundFee(String fundId) {
        return this.fundInformationMapper.getFundFee(fundId);
    }

    public List<ManualFundManagerChangeDo> listFundManagerChangeHistory(String fundId) {
        return this.fundInformationMapper.listFundManagerChangeHistory(fundId);
    }

    public List<MfManagerFundNumDo> listMfManagerFundNumByManagerId(List<String> managerIdList) {
        return this.fundInformationMapper.listMfManagerFundNumByManagerId(managerIdList);
    }

    public String getFundLastAssetSize(String fundId) {
        BigDecimal assetSize = this.fundAssetSizeMapper.getFundLastAssetSize(fundId);
        return assetSize == null ? null : assetSize.toPlainString();
    }

    public ValueLabelVO getLastCompanyScale(String companyId) {
        return this.companyFundScaleRangeHistoryMapper.getLastCompanyScale(companyId);
    }
}
