package com.smppw.analysis.domain.dao.nav;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dataobject.nav.BaseNavDO;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.common.pojo.enums.Frequency;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/8/11 10:23
 * @description 抽象的净值查询接口，目前不加缓存（考虑在什么位置加缓存，以什么结构存储）
 */
public abstract class AbstractNavDao<NAV extends BaseNavDO> implements NavDao {
    @Override
    public Map<String, List<NavDto>> getNav(List<String> secIds, String startDate, String endDate,
                                            Integer curveTypeId, Integer strategyId, Map<String, Frequency> secFrequencyMap) {
        if (CollUtil.isEmpty(secIds)) {
            return MapUtil.empty();
        }
        // 去重并保证顺序
        List<String> refIds = ListUtil.list(true);
        for (String secId : secIds) {
            if (!refIds.contains(secId)) {
                refIds.add(secId);
            }
        }
        List<NAV> navList = this.listSecNav(refIds, startDate, endDate, curveTypeId, strategyId, secFrequencyMap);
        return this.convertToMap(navList);
    }

    /**
     * 获取多个标的的净值序列
     *
     * @param secIds          标的id集合
     * @param startDate       开始日期
     * @param endDate         结束日期
     * @param curveTypeId     策略曲线id（机构或经理）
     * @param strategyId      策略id（机构或经理）
     * @param secFrequencyMap 标的频率（机构或经理）
     * @return /
     */
    protected abstract List<NAV> listSecNav(List<String> secIds, String startDate, String endDate,
                                            Integer curveTypeId, Integer strategyId, Map<String, Frequency> secFrequencyMap);

    /**
     * 多标的净值序列分组
     *
     * @param navList 待分组的净值序列
     * @return /
     */
    private Map<String, List<NavDto>> convertToMap(List<NAV> navList) {
        if (CollUtil.isEmpty(navList)) {
            return MapUtil.empty();
        }
        Map<String, List<NavDto>> navMap = navList.stream().filter(e -> e.getNavDate() != null).map(e -> {
            NavDto navDto = new NavDto();
            navDto.setFundId(e.getRefId());
            navDto.setPriceDate(e.getNavDate());
            navDto.setOriginalNav(e.getNav());
            navDto.setCumulativeNav(e.getCumulativeNav());
            navDto.setCumulativeNavWithdrawal(e.getCumulativeNavWithdrawal());
            return navDto;
        }).collect(Collectors.groupingBy(NavDto::getFundId));
        navMap.forEach((k, v) -> v.sort(Comparator.comparing(NavDto::getPriceDate)));
        return navMap;
    }
}
