package com.smppw.analysis.domain.dao.nav;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dataobject.nav.CompanyFittedCurveDo;
import com.smppw.analysis.domain.dataobject.nav.CompanyFittedCurveWeeklyDo;
import com.smppw.analysis.domain.dataobject.nav.CompanyNavDO;
import com.smppw.analysis.domain.mapper.core.CompanyFittedCurveMapper;
import com.smppw.analysis.domain.mapper.core.CompanyFittedCurveWeeklyMapper;
import com.smppw.common.cache.CaffeineLocalCache;
import com.smppw.common.pojo.enums.Frequency;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component(NavConstants.COMPANY_NAV)
public class CompanyNavDao extends AbstractNavDao<CompanyNavDO> {
    private final CompanyFittedCurveMapper mapper;
    private final CompanyFittedCurveWeeklyMapper weeklyMapper;

    public CompanyNavDao(CompanyFittedCurveMapper mapper, CompanyFittedCurveWeeklyMapper weeklyMapper) {
        this.mapper = mapper;
        this.weeklyMapper = weeklyMapper;
    }

    @Override
    protected List<CompanyNavDO> listSecNav(List<String> secIds, String startDate, String endDate, Integer curveTypeId, Integer strategyId, Map<String, Frequency> secFrequencyMap) {
        Map<Boolean, List<String>> monthWeekMap = secIds.stream().collect(Collectors.groupingBy(e -> Frequency.Weekly == secFrequencyMap.get(e)));
        List<String> monthSecIdList = monthWeekMap.getOrDefault(Boolean.FALSE, ListUtil.empty());
        List<String> weekSecIdList = monthWeekMap.getOrDefault(Boolean.TRUE, ListUtil.empty());
        List<CompanyNavDO> resultList = ListUtil.list(true);
        if (monthSecIdList.size() > 0) {
            String startDateMonth = null;
            if (StringUtils.isNotEmpty(startDate) && startDate.length() >= 7) {
                startDateMonth = startDate.substring(0, 7);
            }
            List<CompanyFittedCurveDo> tempList = this.mapper.listNavByTimeInterval(monthSecIdList, startDateMonth, endDate, curveTypeId, strategyId, false);
            if (CollUtil.isNotEmpty(tempList)) {
                resultList.addAll(tempList);
            }
        }
        if (weekSecIdList.size() > 0) {
            String startYearWeek = null;
            String endYearWeek = null;
            if (StringUtils.isNotEmpty(startDate)) {
                startYearWeek = CaffeineLocalCache.getYearWeekByDate(startDate);
            }
            if (StringUtils.isNotEmpty(endDate)) {
                endYearWeek = CaffeineLocalCache.getYearWeekByDate(endDate);
            }
            List<CompanyFittedCurveWeeklyDo> tempList = this.weeklyMapper.listNavByTimeInterval(weekSecIdList, startYearWeek, endYearWeek, curveTypeId, strategyId, false);
            if (CollUtil.isNotEmpty(tempList)) {
                resultList.addAll(tempList);
            }
        }
        return resultList;
    }
}
