package com.smppw.analysis.domain.dataobject;

import lombok.Getter;
import lombok.Setter;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 公司基本信息表
 */
@Setter
@Getter
public class CompanyInformationDo {
    /**
     * 公司id
     */
    private String companyId;

    /**
     * 上级公司ID
     */
    private String pCompanyId;

    /**
     * 公司中文全称
     */
    private String companyName;

    /**
     * 公司中文简称
     */
    private String companyShortName;

    /**
     * 中基协-公司中文全称
     */
    private String amacCompanyName;

    /**
     * 中基协链接
     */
    private String amacUrl;

    /**
     * 公司类型：1-私募证券投资、2-公募基金公司、3-银行、4-证券公司、5-信托公司、6-审计机构，7-法律顾问，8-投资顾问,9-行政管理人,10-上市公司，11-期货公司，12-基金公司子公司，13工作室，14-私募股权投资，15-证券公司子公司，16-期货公司子公司，17-私募创业投资，18-私募其他投资,19-保险公司,20-保险公司子公司,21-私募资产配置类管理人,22-证券公司私募基金子公司,23-独立第三方销售机构,24-支付结算机构,25-独立服务机构,26-地方自律组织,-1-其他
     */
    private Integer companyType;

    /**
     * 公司成立日期
     */
    private Date establishDate;

    /**
     * 公司注册国家：1-中国大陆、2-香港、3-新加坡、4-开曼群岛、5-台湾、6-英属维尔京群岛BVI、-1-其他
     */
    private Integer domicileId;

    /**
     * 公司注册地址
     */
    private String companyAddress;

    /**
     * 办公地址
     */
    private String companyAddress2;

    /**
     * 城市ID，fund_parameters表外键
     */
    private String cityId;

    /**
     * 办公城市id
     */
    private String officeCityId;

    /**
     * 国际电话区号
     */
    private String countryCode;

    /**
     * 联系电话
     */
    private String contactPhone;

    /**
     * 邮政编码
     */
    private String postCode;

    /**
     * 传真
     */
    private String fax;

    /**
     * 电子邮箱
     */
    private String email;

    /**
     * 公司网址
     */
    private String website;

    /**
     * 公司注册资本，<量纲：万元>
     */
    private BigDecimal registeredCapital;

    /**
     * 实缴资本(万元)
     */
    private BigDecimal paidCapital;

    /**
     * 注册资本实缴比例
     */
    private BigDecimal capitalPaidRate;

    /**
     * 员工人数
     */
    private Integer employeeCnts;

    /**
     * 是否会员：1-是，0-否
     */
    private Byte isMember;

    /**
     * 会员类型：1-普通会员、2-联席会员、3-观察会员、4-特别会员
     */
    private Byte memberType;

    /**
     * 中基协会员编号
     */
    private String memberCode;

    /**
     * 入会时间
     */
    private Date joinDate;

    /**
     * 法律意见书状态
     */
    private String legalOpinionStatus;

    /**
     * 特别提示信息
     */
    private String specialTips;

    /**
     * 执业牌照名(即将废弃,统一至credit_code)
     */
    private String licence;

    /**
     * 公司简介
     */
    private String companyProfile;

    /**
     * 是否有海外基金，0-否，1-是，-1-其他
     */
    private Integer offshoreFund;

    /**
     * 投研团队简介
     */
    private String teamProfile;

    /**
     * 投资理念
     */
    private String philosopy;

    /**
     * 公司状态：1-运行 2-注销 3-吊销
     */
    private Integer companyStatus;

    /**
     * 公司注册资本货币单位：1-人民币，2-港币，3-美元，-1-其他
     */
    private Integer baseCurrencyCrc;

    /**
     * 企业性质:1 中外合作企业  2中外合资企业 3内资企业  4境外机构 5外商独资企业 6政府机构
     */
    private Byte natureBusinessType;

    /**
     * 公司是否在前台可见
     */
    private Integer isvisible;

    /**
     * 创建者Id,默认第一次创建者名称，创建后不变更
     */
    private Integer creatorid;

    /**
     * 创建时间,默认第一次创建的getdate()时间
     */
    private Date createtime;

    /**
     * 修改者Id;第一次创建时与Creator值相同，修改时与修改人值相同
     */
    private Integer updaterid;

    /**
     * 修改时间;第一次创建时与CreatTime值相同，修改时与修改时间相同
     */
    private Date updatetime;

    /**
     * 记录的有效性；1-有效；0-无效；
     */
    private Integer isvalid;

    private String registerNumber;

    private String registerNumberAddress;

    /**
     * 公司logo
     */
    private String logo;

    /**
     * 公司资产规模 1:0-5亿 2:5-10亿 3:10-20亿 4:20-50亿 5:50-100亿 6:100亿以上
     */
    private Byte companyAssetSize;

    /**
     * 规模范围日期(对应company_asset_size)
     */
    private Date fundscaleDate;

    /**
     * 备案日期
     */
    private Date registerDate;

    /**
     * 保证函图片地址
     */
    private String commitment;

    /**
     * 管理规模(确数)
     */
    private BigDecimal fundscale;

    /**
     * 基金管理人全称（英文）
     */
    private String companyEnglishName;

    /**
     * 组织机构代码
     */
    private String organizationNumber;

    /**
     * 业务类型
     */
    private String businessType;

    /**
     * 是否有投顾资质：1-是；0-否
     */
    private Integer hasConsultingQualification;

    /**
     * 是否有投顾资质(非中基协)：1-是；0-否
     */
    private Integer hasConsultingQualificationNotAmac;

    /**
     * 备案状态：0-未备案,1-备案注销,2-备案存续
     */
    private Byte registerStatus;

    /**
     * 备案注销日期
     */
    private Date cancelDate;

    /**
     * 管理基金主要类别（与中基协同步）
     */
    private String fundCategoryOri;

    /**
     * 信用代码
     */
    private String creditCode;

    /**
     * 上市地点:1-上海 2-深圳
     */
    private Byte location;

    /**
     * 经营范围
     */
    private String businessScope;

    /**
     * 公司曾用名
     */
    private String formerName;

    /**
     * 吊销日期
     */
    private Date revokeDate;

    /**
     * 注销类型: 1-主动注销 2-依公告注销 3-协会注销
     */
    private Byte cancelType;

    /**
     * 当前是否备案展示：1-是 0-否 -1-不详
     */
    private Byte isAmacShow;
}