package com.smppw.analysis.domain.manager.info.handler;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.smppw.analysis.domain.dao.FundInformationDao;
import com.smppw.analysis.domain.dao.nav.NavFactory;
import com.smppw.analysis.domain.dataobject.MonetaryFundProfitDO;
import com.smppw.analysis.domain.dataobject.info.FundInfoDO;
import com.smppw.analysis.domain.dto.info.FundHeadInfoVO;
import com.smppw.analysis.domain.dto.info.PubliclyCurrencyFundHeadInfoVO;
import com.smppw.analysis.domain.dto.info.PubliclyFundHeadInfoVO;
import com.smppw.analysis.domain.manager.info.AbstractFundHeadInfo;
import com.smppw.analysis.domain.manager.info.HeadInfoConstants;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.enums.strategy.Strategy;
import com.smppw.utils.BigDecimalUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Objects;

@Component(HeadInfoConstants.PUBLICLY_FUND)
public class PubliclyFundHeadInfo extends AbstractFundHeadInfo<PubliclyFundHeadInfoVO> {
    public PubliclyFundHeadInfo(NavFactory navFactory, FundInformationDao fundInformationDao, BaseInfoService baseInfoService) {
        super(navFactory, fundInformationDao, baseInfoService);
    }

    @Override
    public PubliclyFundHeadInfoVO handleExtInfo(FundInfoDO fundInfo, FundHeadInfoVO headInfo, Map<String, String> secNameMap) {
        PubliclyCurrencyFundHeadInfoVO result = BeanUtil.copyProperties(headInfo, PubliclyCurrencyFundHeadInfoVO.class);
        if (fundInfo.getStrategy() != null) {
            result.setStrategy(secNameMap.get(fundInfo.getStrategy().toString()));
            result.setStrategyId(fundInfo.getStrategy());
        }
        if (fundInfo.getSubstrategy() != null) {
            result.setSubstrategy(secNameMap.get(fundInfo.getSubstrategy().toString()));
            result.setSubstrategyId(fundInfo.getSubstrategy());
        }
        if (Objects.equals(Strategy.Cash.getStrategyId(), fundInfo.getStrategy())) {
            result.setIsCurrency(1);
            List<MonetaryFundProfitDO> tempList = this.fundInformationDao.queryMonetaryFund(fundInfo.getRefId());
            if (CollectionUtil.isNotEmpty(tempList)) {
                MonetaryFundProfitDO monetaryFundProfitDO = tempList.get(tempList.size() - 1);
                result.setProfitSevenDayAnnual(monetaryFundProfitDO.getProfitServenDayAnnual());
                if ("万份".equals(result.getPublicWorthUnit())) {
                    result.setRetPerUnit(monetaryFundProfitDO.getProfitPerMillion());
                } else if ("百万份".equals(result.getPublicWorthUnit())) {
                    result.setRetPerUnit(BigDecimalUtils.multiply(monetaryFundProfitDO.getProfitPerMillion(), BigDecimalUtils.toBigDecimal("100")));
                } else if ("百份".equals(result.getPublicWorthUnit())) {
                    result.setRetPerUnit(BigDecimalUtils.divide(monetaryFundProfitDO.getProfitPerMillion(), BigDecimalUtils.toBigDecimal("100")));
                }
                result.setRetPerUnit(monetaryFundProfitDO.getProfitPerMillion());
            }
        } else {
            result.setIsCurrency(0);
        }
        return result;
    }
}
