package com.smppw.analysis.domain.service.impl;

import com.smppw.analysis.domain.dao.nav.NavConstants;
import com.smppw.analysis.domain.dao.nav.NavFactory;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.common.pojo.enums.NavType;
import com.smppw.common.pojo.enums.Visibility;
import com.smppw.constants.SecType;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class NavServiceImpl implements NavService {
    private final NavFactory navFactory;
    private final BaseInfoService baseInfoService;

    public NavServiceImpl(NavFactory navFactory, BaseInfoService baseInfoService) {
        this.navFactory = navFactory;
        this.baseInfoService = baseInfoService;
    }

    @Override
    public Map<String, List<DateValue>> getSecIdDateValueNavListMapFromRedisAndDB(List<String> mainSecIdList, List<String> benchmarkIdList, List<String> indexList,
                                                                                  String startDate, String endDate, Integer curveTypeId, Integer strategyId, NavType navType,
                                                                                  Visibility visibility, Map<String, Frequency> secFrequencyMap) {
        //全部的标的集合
        List<String> secIds = new ArrayList<>();
        secIds.addAll(mainSecIdList);
        secIds.addAll(benchmarkIdList);
        secIds.addAll(indexList);
        return getSecIdDateValueNavListMapByDb(secIds, startDate, endDate, curveTypeId, strategyId, visibility, navType, secFrequencyMap);
    }

    @Override
    public Map<String, List<DateValue>> getSecIdDateValueNavListMapByDb(List<String> allSecIdList, String startDate, String endDate,
                                                                        Integer curveTypeId, Integer strategyId, Visibility visibility, NavType navType, Map<String, Frequency> secFrequencyMap) {

        Map<String, List<DateValue>> allNavMap = new HashMap<>();

        //对secId进行分类
        Map<String, List<String>> secIdTypeMap = baseInfoService.getTypeSecMap(allSecIdList);

        //1.市场私募净值
        List<String> marketPrivateFundIdList = secIdTypeMap.get(SecType.PRIVATELY_OFFERED_FUND);
        if (marketPrivateFundIdList != null && marketPrivateFundIdList.size() > 0) {
            Map<String, List<DateValue>> privatelyOfferedFundNavMap = getPrivatelyOfferedFundNavMap(marketPrivateFundIdList, startDate, endDate, navType, visibility);
            allNavMap.putAll(privatelyOfferedFundNavMap);
        }

        //2.市场公募净值
        List<String> marketPublicFundIdList = secIdTypeMap.get(SecType.PUBLICLY_OFFERED_FUNDS);
        if (marketPublicFundIdList != null && marketPublicFundIdList.size() > 0) {
            Map<String, List<DateValue>> marketPubliclyOfferedFundNavMap = getPubliclyOfferedFundNavMap(marketPublicFundIdList, startDate, endDate, navType, visibility);
            allNavMap.putAll(marketPubliclyOfferedFundNavMap);
        }

        //5.市场指数行情
        List<String> marketIndexIdList = secIdTypeMap.get(SecType.INDEX_FUND);
        if (marketIndexIdList != null && marketIndexIdList.size() > 0) {
            Map<String, List<DateValue>> marketIndexNavMap = getMarketIndexNavMap(marketIndexIdList, startDate, endDate, navType);
            allNavMap.putAll(marketIndexNavMap);
        }

        //7.融智指数行情
        List<String> rongzhiIndexIdList = secIdTypeMap.get(SecType.RONGZHI_INDEX);
        if (rongzhiIndexIdList != null && rongzhiIndexIdList.size() > 0) {
            Map<String, List<DateValue>> rongzhiIndexIdNavMap = getRongzhiIndexNavMap(rongzhiIndexIdList, startDate, endDate, navType);
            allNavMap.putAll(rongzhiIndexIdNavMap);
        }

        //10.机构拟合净值
        List<String> companyIdList = secIdTypeMap.get(SecType.COMPANY);
        if (companyIdList != null && companyIdList.size() > 0) {
            Map<String, List<DateValue>> companyIdNavMap = getCompanyNavMap(companyIdList, startDate, endDate, curveTypeId, strategyId, navType, secFrequencyMap);
            allNavMap.putAll(companyIdNavMap);
        }

        //11.经理拟合净值
        List<String> managerIdList = secIdTypeMap.get(SecType.MANAGER);
        if (managerIdList != null && managerIdList.size() > 0) {
            Map<String, List<DateValue>> managerIdNavMap = getManagerNavMap(managerIdList, startDate, endDate, curveTypeId, strategyId, navType, secFrequencyMap);
            allNavMap.putAll(managerIdNavMap);
        }

        //13.无风险收益
        List<String> riskOfFree = secIdTypeMap.get(SecType.RISK_OF_FREE);
        if (riskOfFree != null && riskOfFree.size() > 0) {
            Map<String, List<DateValue>> riskOfFreeNavMap = getRiskOfFreeNavMap(riskOfFree, startDate, endDate, navType);
            allNavMap.putAll(riskOfFreeNavMap);
        }

        return allNavMap;
    }

    private Map<String, List<DateValue>> getRiskOfFreeNavMap(List<String> riskOfFree, String startDate, String endDate, NavType navType) {
//        Map<String, List<NavDto>> riskOfFreeNavMap = riskOfFreeNavDao.getNav(riskOfFree, startDate, endDate, false);
        Map<String, List<NavDto>> riskOfFreeNavMap = this.navFactory.getInstance(NavConstants.RISK_INDEX_NAV).getNav(riskOfFree, startDate, endDate);
        return SecNavDtoListMapToSecDateValueListMap(navType, riskOfFreeNavMap);
    }

    @Override
    public Map<String, List<DateValue>> getManagerNavMap(List<String> managerIdList, String startDate, String endDate, Integer curveType, Integer strategy, NavType navType, Map<String, Frequency> secFrequencyMap) {
//        Map<String, List<NavDto>> managerIdNavDtoListMap = managerNavDao.getNav(managerIdList, startDate, endDate, curveType, strategy, secFrequencyMap, false);
        Map<String, List<NavDto>> managerIdNavDtoListMap = this.navFactory.getInstance(NavConstants.MANAGER_NAV).getNav(managerIdList, startDate, endDate, curveType, strategy, secFrequencyMap);
        return SecNavDtoListMapToSecDateValueListMap(navType, managerIdNavDtoListMap);
    }

    @Override
    public Map<String, List<DateValue>> getCompanyNavMap(List<String> companyIdList, String startDate, String endDate, Integer curveType, Integer strategy, NavType navType, Map<String, Frequency> secFrequencyMap) {
//        Map<String, List<NavDto>> companyIdNavDtoListMap = companyNavDao.getNav(companyIdList, startDate, endDate, curveType, strategy, secFrequencyMap, false);
        Map<String, List<NavDto>> companyIdNavDtoListMap = this.navFactory.getInstance(NavConstants.COMPANY_NAV).getNav(companyIdList, startDate, endDate, curveType, strategy, secFrequencyMap);
        return SecNavDtoListMapToSecDateValueListMap(navType, companyIdNavDtoListMap);
    }

    @Override
    public Map<String, List<DateValue>> getRongzhiIndexNavMap(List<String> rongzhiIndexIdList, String startDate, String endDate, NavType navType) {
//        Map<String, List<NavDto>> rongzhiIndexIdNavDtoListMap = rongzhiIndexNavDao.getNav(rongzhiIndexIdList, startDate, endDate, false);
        Map<String, List<NavDto>> rongzhiIndexIdNavDtoListMap = this.navFactory.getInstance(NavConstants.PPW_INDEX_NAV).getNav(rongzhiIndexIdList, startDate, endDate);
        return SecNavDtoListMapToSecDateValueListMap(navType, rongzhiIndexIdNavDtoListMap);
    }

    @Override
    public Map<String, List<DateValue>> getMarketIndexNavMap(List<String> marketIndexIdList, String startDate, String endDate, NavType navType) {
//        Map<String, List<NavDto>> indexIdNavDtoListMap = marketIndexDao.getNav(marketIndexIdList, startDate, endDate, false);
        Map<String, List<NavDto>> indexIdNavDtoListMap = this.navFactory.getInstance(NavConstants.MARKET_INDEX_NAV).getNav(marketIndexIdList, startDate, endDate);
        return SecNavDtoListMapToSecDateValueListMap(navType, indexIdNavDtoListMap);
    }

    @Override
    public Map<String, List<DateValue>> getPubliclyOfferedFundNavMap(List<String> fundIdList, String startDate, String endDate, NavType navType, Visibility visibility) {
//        Map<String, List<NavDto>> fundIdNavDtoListMap = publiclyOfferedFundNavDao.getNav(fundIdList, startDate, endDate, navType, false);
        Map<String, List<NavDto>> fundIdNavDtoListMap = this.navFactory.getInstance(NavConstants.PUBLICLY_FUND_NAV).getNav(fundIdList, startDate, endDate);
        return SecNavDtoListMapToSecDateValueListMap(navType, fundIdNavDtoListMap);
    }

    @Override
    public Map<String, List<DateValue>> getPrivatelyOfferedFundNavMap(List<String> fundIdList, String startDate, String endDate, NavType navType, Visibility visibility) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = this.navFactory.getInstance(NavConstants.PRIVATELY_FUND_NAV).getNav(fundIdList, startDate, endDate);
        return SecNavDtoListMapToSecDateValueListMap(navType, fundIdNavDtoListMap);
    }

    private Map<String, List<DateValue>> SecNavDtoListMapToSecDateValueListMap(NavType navType, Map<String, List<NavDto>> secIdNavDtoListMap) {
        Map<String, List<DateValue>> secIdDateValueListMap = new HashMap<>();
        for (Map.Entry<String, List<NavDto>> fundIdNavDtoListEntry : secIdNavDtoListMap.entrySet()) {
            List<NavDto> navDtoList = fundIdNavDtoListEntry.getValue();
            String fundId = fundIdNavDtoListEntry.getKey();
            List<DateValue> dateValueList = this.convertNavList(navDtoList, navType);
            secIdDateValueListMap.put(fundId, dateValueList);
        }
        return secIdDateValueListMap;
    }

    private List<DateValue> convertNavList(List<NavDto> navDtoList, NavType navType) {
        List<DateValue> dateValueList = new ArrayList<>();
        for (NavDto navDto : navDtoList) {
            DateValue dateValue = new DateValue();
            dateValue.setDate(navDto.getPriceDate());
            if (NavType.WithdrawalNav == navType) {
                dateValue.setValue(navDto.getCumulativeNavWithdrawal() != null ? navDto.getCumulativeNavWithdrawal().doubleValue() : null);
            } else if (NavType.OriginalNav == navType) {
                dateValue.setValue(navDto.getOriginalNav() != null ? navDto.getOriginalNav().doubleValue() : null);
            } else if (NavType.All == navType) {
                dateValue.setValue(navDto.getOriginalNav() != null ? navDto.getOriginalNav().doubleValue() : null);
                dateValue.setOriginalNav(navDto.getOriginalNav() != null ? navDto.getOriginalNav() : null);
                dateValue.setCumulativeNav(navDto.getCumulativeNav() != null ? navDto.getCumulativeNav() : null);
                dateValue.setCumulativeNavWithdrawal(navDto.getCumulativeNavWithdrawal() != null ? navDto.getCumulativeNavWithdrawal() : null);
            } else {
                dateValue.setValue(navDto.getCumulativeNav() != null ? navDto.getCumulativeNav().doubleValue() : null);
            }
            dateValueList.add(dateValue);
        }
        return dateValueList;
    }
}
